@@ -14,9 +14,24 @@
 
 #include "kgpgexternalactions.h"
 
-#include <QDesktopWidget>
-#include <QFont>
-#include <QProcess>
+#include "detailedconsole.h"
+#include "foldercompressjob.h"
+#include "keyservers.h"
+#include "keysmanager.h"
+#include "kgpgfirstassistant.h"
+#include "kgpginterface.h"
+#include "kgpgsettings.h"
+#include "kgpgtextinterface.h"
+#include "selectpublickeydialog.h"
+#include "selectsecretkey.h"
+#include "core/images.h"
+#include "editor/kgpgeditor.h"
+#include "editor/kgpgtextedit.h"
+#include "transactions/kgpgdecrypt.h"
+#include "transactions/kgpgencrypt.h"
+#include "transactions/kgpgsigntext.h"
+#include "transactions/kgpgtransactionjob.h"
+#include "transactions/kgpgverify.h"
 
 #include <KActionCollection>
 #include <KMessageBox>
@@ -25,26 +40,13 @@
 #include <KToolInvocation>
 #include <KUniqueApplication>
 #include <KZip>
-#include <kio/renamedialog.h>
+#include <QDesktopWidget>
+#include <QFont>
+#include <QProcess>
 #include <kio/global.h>
+#include <kio/renamedialog.h>
 #include <kjobtrackerinterface.h>
 
-#include "images.h"
-#include "keyservers.h"
-#include "keysmanager.h"
-#include "kgpgeditor.h"
-#include "kgpgfirstassistant.h"
-#include "kgpglibrary.h"
-#include "kgpgsettings.h"
-#include "kgpgtextinterface.h"
-#include "kgpgtextedit.h"
-#include "selectpublickeydialog.h"
-#include "selectsecretkey.h"
-#include "kgpginterface.h"
-#include "transactions/kgpgdecrypt.h"
-#include "foldercompressjob.h"
-#include "kgpgencrypt.h"
-
 KGpgExternalActions::KGpgExternalActions(KeysManager *parent, KGpgItemModel *model)
 	: QObject(parent),
 	m_model(model),
@@ -57,63 +59,106 @@ KGpgExternalActions::~KGpgExternalActions()
 {
 }
 
-void KGpgExternalActions::encryptDroppedFile()
+void KGpgExternalActions::slotEncryptDroppedFiles(const KUrl::List &urls)
 {
+	Q_ASSERT(!urls.isEmpty());
+
+	KgpgSelectPublicKeyDlg *dialog = new KgpgSelectPublicKeyDlg(0, m_model, goDefaultKey(), false, urls);
+	connect(dialog, SIGNAL(accepted()), SLOT(slotEncryptionKeySelected()));
+	connect(dialog, SIGNAL(rejected()), dialog, SLOT(deleteLater()));
+	dialog->show();
+}
+
+void KGpgExternalActions::slotEncryptionKeySelected()
+{
+	KgpgSelectPublicKeyDlg *dialog = qobject_cast<KgpgSelectPublicKeyDlg *>(sender());
+	Q_ASSERT(dialog != NULL);
+	sender()->deleteLater();
+
 	QStringList opts;
-	KgpgLibrary *lib = new KgpgLibrary(0);
-	connect(lib, SIGNAL(systemMessage(QString)), SLOT(busyMessage(QString)));
+	QString defaultKey;
 
 	if (KGpgSettings::encryptFilesTo()) {
 		if (KGpgSettings::pgpCompatibility())
 			opts << QLatin1String( "--pgp6" );
 
-		lib->slotFileEnc(droppedUrls, opts, m_model, goDefaultKey(), KGpgSettings::fileEncryptionKey());
-	} else {
-		lib->slotFileEnc(droppedUrls, opts, m_model, goDefaultKey());
+		defaultKey = KGpgSettings::fileEncryptionKey();
 	}
+
+	KGpgEncrypt::EncryptOptions eopt = KGpgEncrypt::DefaultEncryption;
+
+	if (dialog->getUntrusted())
+		eopt |= KGpgEncrypt::AllowUntrustedEncryption;
+	if (dialog->getArmor())
+		eopt |= KGpgEncrypt::AsciiArmored;
+	if (dialog->getHideId())
+		eopt |= KGpgEncrypt::HideKeyId;
+
+	if (KGpgSettings::allowCustomEncryptionOptions()) {
+		const QString customopts(dialog->getCustomOptions().isEmpty());
+
+		if (!customopts.isEmpty())
+			opts << customopts.split(QLatin1Char(' '), QString::SkipEmptyParts);
+	}
+
+	QStringList keys(dialog->selectedKeys());
+	if (!defaultKey.isEmpty() && !keys.contains(defaultKey))
+		keys.append(defaultKey);
+
+	if (dialog->getSymmetric())
+		keys.clear();
+
+	KGpgEncrypt *enc = new KGpgEncrypt(parent(), keys, dialog->getFiles(), eopt, opts);
+	KGpgTransactionJob *encjob = new KGpgTransactionJob(enc);
+
+	KIO::getJobTracker()->registerJob(encjob);
+	encjob->start();
 }
 
-void KGpgExternalActions::encryptDroppedFolder()
+void KGpgExternalActions::encryptDroppedFolders(const KUrl::List &urls)
 {
 	compressionScheme = 0;
-	kgpgfoldertmp = new KTemporaryFile();
-	kgpgfoldertmp->open();
 
-	if (KMessageBox::Cancel == KMessageBox::warningContinueCancel(m_keysmanager,
-				i18n("<qt>KGpg will now create a temporary archive file:<br /><b>%1</b> to process the encryption. The file will be deleted after the encryption is finished.</qt>",
-				kgpgfoldertmp->fileName()), i18n("Temporary File Creation"), KStandardGuiItem::cont(),
-				KStandardGuiItem::cancel(), QLatin1String( "FolderTmpFile" ))) {
-		delete kgpgfoldertmp;
-		kgpgfoldertmp = NULL;
+	KTemporaryFile *tmpfolder = new KTemporaryFile();
+
+	if (!tmpfolder->open()) {
+		delete tmpfolder;
+		KMessageBox::sorry(m_keysmanager, i18n("Cannot create temporary file for folder compression."), i18n("Temporary File Creation"));
 		return;
 	}
 
-	dialog = new KgpgSelectPublicKeyDlg(m_keysmanager, m_model, goDefaultKey(), false, droppedUrls);
+	if (KMessageBox::Continue != KMessageBox::warningContinueCancel(m_keysmanager,
+				i18n("<qt>KGpg will now create a temporary archive file:<br /><b>%1</b> to process the encryption. The file will be deleted after the encryption is finished.</qt>",
+				tmpfolder->fileName()), i18n("Temporary File Creation"), KStandardGuiItem::cont(),
+				KStandardGuiItem::cancel(), QLatin1String( "FolderTmpFile" ))) {
+		delete tmpfolder;
+		return;
+	}
+
+	m_kgpgfoldertmp = tmpfolder;
+
+	KgpgSelectPublicKeyDlg *dialog = new KgpgSelectPublicKeyDlg(m_keysmanager, m_model, goDefaultKey(), false, urls);
 
 	KHBox *bGroup = new KHBox(dialog->optionsbox);
 
 	(void) new QLabel(i18n("Compression method for archive:"), bGroup);
 
 	KComboBox *optionbx = new KComboBox(bGroup);
-	optionbx->addItem(i18n("Zip"));
-	optionbx->addItem(i18n("Gzip"));
-	optionbx->addItem(i18n("Bzip2"));
-	optionbx->addItem(i18n("Tar"));
-	optionbx->addItem(i18n("Tar/XZ"));
+	foreach (const QString &aname, FolderCompressJob::archiveNames())
+		optionbx->addItem(aname);
 
 	connect(optionbx, SIGNAL(activated(int)), SLOT(slotSetCompression(int)));
-	connect(dialog, SIGNAL(okClicked()), SLOT(startFolderEncode()));
-	connect(dialog, SIGNAL(cancelClicked()), SLOT(slotAbortEnc()));
+	connect(dialog, SIGNAL(accepted()), SLOT(startFolderEncode()));
+	connect(dialog, SIGNAL(rejected()), SLOT(slotAbortEnc()));
 
 	dialog->exec();
 }
 
 void KGpgExternalActions::slotAbortEnc()
 {
-	dialog->deleteLater();
-	dialog = NULL;
-	delete kgpgfoldertmp;
-	kgpgfoldertmp = NULL;
+	sender()->deleteLater();
+	delete m_kgpgfoldertmp;
+	m_kgpgfoldertmp = NULL;
 }
 
 void KGpgExternalActions::slotSetCompression(int cp)
@@ -123,6 +168,12 @@ void KGpgExternalActions::slotSetCompression(int cp)
 
 void KGpgExternalActions::startFolderEncode()
 {
+	KgpgSelectPublicKeyDlg *dialog = qobject_cast<KgpgSelectPublicKeyDlg *>(sender());
+	Q_ASSERT(dialog != NULL);
+	dialog->deleteLater();
+
+	const KUrl::List urls = dialog->getFiles();
+
 	QStringList selec = dialog->selectedKeys();
 	KGpgEncrypt::EncryptOptions encOptions = KGpgEncrypt::DefaultEncryption;
 	const QStringList encryptOptions = dialog->getCustomOptions().split(QLatin1Char(' '),  QString::SkipEmptyParts);
@@ -131,28 +182,8 @@ void KGpgExternalActions::startFolderEncode()
 	} else {
 		Q_ASSERT(!selec.isEmpty());
 	}
-	QString extension;
 
-	switch (compressionScheme) {
-	case 0:
-		extension = QLatin1String( ".zip" );
-		break;
-	case 1:
-		extension = QLatin1String( ".tar.gz" );
-		break;
-	case 2:
-		extension = QLatin1String( ".tar.bz2" );
-		break;
-	case 3:
-		extension = QLatin1String( ".tar" );
-		break;
-	case 4:
-		extension = QLatin1String( ".tar.xz" );
-		break;
-	default:
-		Q_ASSERT(compressionScheme == 0);
-		return;
-	}
+	QString extension = FolderCompressJob::extensionForArchive(compressionScheme);
 
 	if (dialog->getArmor())
 		extension += QLatin1String( ".asc" );
@@ -168,13 +199,12 @@ void KGpgExternalActions::startFolderEncode()
 	if (dialog->getUntrusted())
 		encOptions |= KGpgEncrypt::AllowUntrustedEncryption;
 
-	KUrl encryptedFile(KUrl::fromPath(droppedUrls.first().path(KUrl::RemoveTrailingSlash) + extension));
+	KUrl encryptedFile(KUrl::fromPath(urls.first().path(KUrl::RemoveTrailingSlash) + extension));
 	QFile encryptedFolder(encryptedFile.path());
 	dialog->hide();
 	if (encryptedFolder.exists()) {
 		QPointer<KIO::RenameDialog> over = new KIO::RenameDialog(m_keysmanager, i18n("File Already Exists"), KUrl(), encryptedFile, KIO::M_OVERWRITE);
 		if (over->exec() == QDialog::Rejected) {
-			dialog->deleteLater();
 			dialog = NULL;
 			delete over;
 			return;
@@ -183,15 +213,10 @@ void KGpgExternalActions::startFolderEncode()
 		delete over;
 	}
 
-	FolderCompressJob *trayinfo = new FolderCompressJob(m_keysmanager, droppedUrls, encryptedFile, kgpgfoldertmp, selec, encryptOptions, encOptions, compressionScheme);
+	FolderCompressJob *trayinfo = new FolderCompressJob(m_keysmanager, urls, encryptedFile, m_kgpgfoldertmp, selec, encryptOptions, encOptions, compressionScheme);
 	connect(trayinfo, SIGNAL(result(KJob*)), SLOT(slotFolderFinished(KJob*)));
 	KIO::getJobTracker()->registerJob(trayinfo);
 	trayinfo->start();
-
-	dialog->accept();
-	dialog->deleteLater();
-	dialog = NULL;
-	kgpgfoldertmp = NULL;
 }
 
 void KGpgExternalActions::slotFolderFinished(KJob *job)
@@ -199,108 +224,130 @@ void KGpgExternalActions::slotFolderFinished(KJob *job)
 	FolderCompressJob *trayinfo = qobject_cast<FolderCompressJob *>(job);
 	Q_ASSERT(trayinfo != NULL);
 
-	delete kgpgfoldertmp;
-	kgpgfoldertmp = NULL;
+	delete m_kgpgfoldertmp;
+	m_kgpgfoldertmp = NULL;
 	if (trayinfo->error())
 		KMessageBox::sorry(m_keysmanager, trayinfo->errorString());
 }
 
-void KGpgExternalActions::busyMessage(const QString &mssge)
-{
-	if (!mssge.isEmpty()) {
-#ifdef __GNUC__
-#warning FIXME: this need to be ported
-#endif
-// 		trayIcon->setToolTip(mssge);
-	}
-}
-
-void KGpgExternalActions::encryptFiles(KUrl::List urls)
-{
-	droppedUrls = urls;
-	encryptDroppedFile();
-}
-
-void KGpgExternalActions::slotVerifyFile()
+void KGpgExternalActions::verifyFile(KUrl url)
 {
 	// check file signature
-	if (droppedUrl.isEmpty())
+	if (url.isEmpty())
 		return;
 
 	QString sigfile;
 	// try to find detached signature.
-	if (!droppedUrl.fileName().endsWith(QLatin1String(".sig"))) {
-		sigfile = droppedUrl.path() + QLatin1String( ".sig" );
+	if (!url.fileName().endsWith(QLatin1String(".sig"))) {
+		sigfile = url.path() + QLatin1String( ".sig" );
 		QFile fsig(sigfile);
 		if (!fsig.exists()) {
-			sigfile = droppedUrl.path() + QLatin1String( ".asc" );
+			sigfile = url.path() + QLatin1String( ".asc" );
 			QFile fsig(sigfile);
 			// if no .asc or .sig signature file included, assume the file is internally signed
 			if (!fsig.exists())
 				sigfile.clear();
 		}
 	} else {
-		sigfile = droppedUrl.path();
-		droppedUrl = KUrl(sigfile.left(sigfile.length() - 4));
+		sigfile = url.path();
+		url = KUrl(sigfile.left(sigfile.length() - 4));
 	}
 
-	// pipe gpg command
-	KGpgTextInterface *verifyFileProcess = new KGpgTextInterface(this);
-	connect (verifyFileProcess, SIGNAL(verifyquerykey(QString)), SLOT(importSignature(QString)));
-	verifyFileProcess->KgpgVerifyFile(droppedUrl, KUrl(sigfile));
+	KGpgVerify *kgpv = new KGpgVerify(parent(), KUrl::List(sigfile));
+	connect(kgpv, SIGNAL(done(int)), SLOT(slotVerificationDone(int)));
+	kgpv->start();
 }
 
-void KGpgExternalActions::importSignature(const QString &ID)
+void KGpgExternalActions::slotVerificationDone(int result)
 {
-	KeyServer *kser = new KeyServer(0, m_model);
-	kser->slotSetText(ID);
-	kser->slotImport();
-}
+	KGpgVerify *kgpv = qobject_cast<KGpgVerify *>(sender());
+	Q_ASSERT(kgpv != NULL);
+	kgpv->deleteLater();
 
-void KGpgExternalActions::signDroppedFile()
-{
-	// create a detached signature for a chosen file
-	if (droppedUrl.isEmpty())
-		return;
+	if (result == KGpgVerify::TS_MISSING_KEY) {
+		KeyServer *kser = new KeyServer(m_keysmanager, m_model);
+		kser->slotSetText(kgpv->missingId());
+		kser->slotImport();
+	} else {
+		const QStringList messages = kgpv->getMessages();
 
-	// select a private key to sign file --> listkeys.cpp
-	QPointer<KgpgSelectSecretKey> opts = new KgpgSelectSecretKey(0, m_model, false);
-	if (opts->exec() != QDialog::Accepted) {
-		delete opts;
-		return;
+		if (messages.isEmpty())
+			return;
+
+		QStringList msglist;
+		foreach (QString rawmsg, messages)
+			msglist << rawmsg.replace(QLatin1Char('<'), QLatin1String("&lt;"));
+
+		(void) new KgpgDetailedInfo(m_keysmanager, KGpgVerify::getReport(messages, m_model),
+				msglist.join(QLatin1String("<br/>")),
+				QStringList(), i18nc("Caption of message box", "Verification Finished"));
 	}
+}
+
+void KGpgExternalActions::signDroppedFiles(const KUrl::List &urls)
+{
+	Q_ASSERT(!urls.isEmpty());
+
+	droppedUrls = urls;
+
+	KgpgSelectSecretKey *keydlg = new KgpgSelectSecretKey(0, m_model, false);
+	connect(keydlg, SIGNAL(accepted()), SLOT(slotSignFiles()));
+	connect(keydlg, SIGNAL(rejected()), keydlg, SLOT(deleteLater()));
+	keydlg->show();
+}
+
+void KGpgExternalActions::slotSignFiles()
+{
+	KgpgSelectSecretKey *keydlg = qobject_cast<KgpgSelectSecretKey *>(sender());
+	Q_ASSERT(keydlg != NULL);
+	sender()->deleteLater();
+
+	const QString signKeyID = keydlg->getKeyID();
 
-	const QString signKeyID(opts->getKeyID());
-	delete opts;
 	QStringList Options;
-	if (KGpgSettings::asciiArmor())
+	KGpgSignText::SignOptions sopts = KGpgSignText::DetachedSignature;
+	if (KGpgSettings::asciiArmor()) {
 		Options << QLatin1String( "--armor" );
+		sopts |= KGpgSignText::AsciiArmored;
+	}
 	if (KGpgSettings::pgpCompatibility())
 		Options << QLatin1String( "--pgp6" );
-	Options << QLatin1String( "--detach-sign" );
 
-	KGpgTextInterface signFileProcess;
-	signFileProcess.signFilesBlocking(signKeyID, droppedUrls, Options);
+	if (droppedUrls.count() > 1) {
+		KGpgTextInterface *signFileProcess = new KGpgTextInterface(this, signKeyID, Options);
+		connect(signFileProcess, SIGNAL(fileSignFinished()), SLOT(slotSigningFinished()));
+		signFileProcess->signFiles(droppedUrls);
+	} else {
+		KGpgSignText *signt = new KGpgSignText(this, signKeyID, droppedUrls, sopts);
+		connect(signt, SIGNAL(done(int)), SLOT(slotSigningFinished()));
+		signt->start();
+	}
 }
 
-void KGpgExternalActions::decryptDroppedFile()
+void KGpgExternalActions::slotSigningFinished()
+{
+	sender()->deleteLater();
+}
+
+void KGpgExternalActions::decryptDroppedFiles(const KUrl::List &urls)
 {
 	m_decryptionFailed.clear();
 
-	decryptFile();
+	decryptFile(urls);
 }
 
-void KGpgExternalActions::decryptFile()
+void KGpgExternalActions::decryptFile(KUrl::List urls)
 {
-	if (droppedUrls.isEmpty())
+	if (urls.isEmpty())
 		return;
 
-	while (!droppedUrls.first().isLocalFile()) {
-		showDroppedFile();
-		droppedUrls.pop_front();
+	while (!urls.first().isLocalFile()) {
+		showDroppedFile(urls.takeFirst());
 	}
 
-	QString oldname(droppedUrls.first().fileName());
+	KUrl first = urls.first();
+
+	QString oldname(first.fileName());
 	if (oldname.endsWith(QLatin1String(".gpg"), Qt::CaseInsensitive) ||
 			oldname.endsWith(QLatin1String(".asc"), Qt::CaseInsensitive) ||
 			oldname.endsWith(QLatin1String(".pgp"), Qt::CaseInsensitive))
@@ -308,14 +355,14 @@ void KGpgExternalActions::decryptFile()
 	else
 		oldname.append(QLatin1String( ".clear" ));
 
-	KUrl swapname(droppedUrls.first().directory(KUrl::AppendTrailingSlash) + oldname);
+	KUrl swapname(first.directory(KUrl::AppendTrailingSlash) + oldname);
 	QFile fgpg(swapname.path());
 	if (fgpg.exists()) {
 		QPointer<KIO::RenameDialog> over = new KIO::RenameDialog(m_keysmanager, i18n("File Already Exists"), KUrl(), swapname, KIO::M_OVERWRITE);
 		if (over->exec() != QDialog::Accepted) {
 			delete over;
-			droppedUrls.pop_front();
-			decryptFile();
+			urls.pop_front();
+			decryptFile(urls);
 			return;
 		}
 
@@ -323,6 +370,7 @@ void KGpgExternalActions::decryptFile()
 		delete over;
 	}
 
+	droppedUrls = urls;
 	KGpgDecrypt *decr = new KGpgDecrypt(this, droppedUrls.first(), swapname);
 	connect(decr, SIGNAL(done(int)), SLOT(slotDecryptionDone(int)));
 	decr->start();
@@ -341,7 +389,7 @@ void KGpgExternalActions::slotDecryptionDone(int status)
 	droppedUrls.pop_front();
 
 	if (!droppedUrls.isEmpty()) {
-		decryptFile();
+		decryptFile(droppedUrls);
 	} else if (!m_decryptionFailed.isEmpty()) {
 		KMessageBox::errorList(NULL,
 					i18np("Decryption of this file failed:", "Decryption of these files failed:",
@@ -350,13 +398,13 @@ void KGpgExternalActions::slotDecryptionDone(int status)
 	}
 }
 
-void KGpgExternalActions::showDroppedFile()
+void KGpgExternalActions::showDroppedFile(const KUrl &file)
 {
 	KgpgEditor *kgpgtxtedit = new KgpgEditor(m_keysmanager, m_model, 0);
-	connect(kgpgtxtedit, SIGNAL(encryptFiles(KUrl::List)), SLOT(encryptFiles(KUrl::List)));
+	connect(kgpgtxtedit, SIGNAL(encryptFiles(KUrl::List)), SLOT(slotEncryptDroppedFiles(KUrl::List)));
 	connect(m_keysmanager, SIGNAL(fontChanged(QFont)), kgpgtxtedit, SLOT(slotSetFont(QFont)));
 
-	kgpgtxtedit->m_editor->openDroppedFile(droppedUrls.first(), false);
+	kgpgtxtedit->m_editor->openDroppedFile(file, false);
 
 	kgpgtxtedit->show();
 }
@@ -369,19 +417,11 @@ void KGpgExternalActions::readOptions()
 
 	if (KGpgSettings::firstRun()) {
 		firstRun();
-	} else {
-		QString path(KGpgSettings::gpgConfigPath());
-
-		if (path.isEmpty()) {
-			if (KMessageBox::Yes == KMessageBox::questionYesNo(0,
-					i18n("<qt>You have not set a path to your GnuPG config file.<br />This may cause some surprising results in KGpg's execution.<br />Would you like to start KGpg's assistant to fix this problem?</qt>"),
-					QString(), KGuiItem(i18n("Start Assistant")), KGuiItem(i18n("Do Not Start"))))
-				startAssistant();
-		} else {
-			QStringList groups(KgpgInterface::getGpgGroupNames(path));
-			if (!groups.isEmpty())
-				KGpgSettings::setGroups(groups.join(QLatin1String( "," )));
-		}
+	} else if (KGpgSettings::gpgConfigPath().isEmpty()) {
+		if (KMessageBox::Yes == KMessageBox::questionYesNo(0,
+				i18n("<qt>You have not set a path to your GnuPG config file.<br />This may cause some surprising results in KGpg's execution.<br />Would you like to start KGpg's assistant to fix this problem?</qt>"),
+				QString(), KGuiItem(i18n("Start Assistant")), KGuiItem(i18n("Do Not Start"))))
+			startAssistant();
 	}
 }
 
@@ -440,3 +480,5 @@ KShortcut KGpgExternalActions::goDefaultKey() const
 {
 	return qobject_cast<KAction *>(m_keysmanager->actionCollection()->action(QLatin1String( "go_default_key" )))->shortcut();
 }
+
+#include "kgpgexternalactions.moc"
